import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)

/**
 * Wrapperklasse für einen Text auf der Zeichenfläche.
 * 
 * @author Albert Wiedemann 
 * @version 1.0
 */
public class Text extends Actor
{
    /** x-Position der linken Seite der Grundlinie. */
    private int x;
    /** y-Position der Grundlinie. */
    private int y;
    /** Farbe des Textes. */
    private String farbe;
    /** Sichtbarkeit des Textes. */
    private boolean sichtbar;
    /** Drehwinkel (mathematisch positiver Drehsinn) des Textes in Grad. */
    private int winkel;
    /** Größe des Textes in Punkten. */
    private int textgröße;
    /** Referenz auf das Delegate-Objekt. */
    private Zeichenfenster.TextIntern symbol;
    /** Referenz auf die eigene Welt. */
    private GrafikWelt welt;
    /** Referenz auf eigene Bild. */
    private GreenfootImage img;

    /**
     * Der Konstruktor erzeugt das Delegate-Objekt
     */
    Text ()
    {
        x = 10;
        y = 10;
        farbe = "schwarz";
        sichtbar = true;
        winkel = 0;
        textgröße = 12;
        symbol = null;
        img = new GreenfootImage(100, 20);
        img.setColor(new Color(255, 255, 255, 0));
        img.fill();
        setImage(img);
        super.setRotation(-winkel);
        welt = GrafikWelt.dieWelt;
        symbol = (Zeichenfenster.TextIntern) welt.fenster.SymbolErzeugen(Zeichenfenster.SymbolArt.text);
        symbol.PositionSetzen(x, y);
        symbol.FarbeSetzen(farbe);
        symbol.SichtbarkeitSetzen(false);
        symbol.WinkelSetzen(winkel);
        symbol.TextGrößeSetzen(textgröße);
    }
    
    /**
     * Erzeugt das Delegate-Objekt.
     * @param world die Welt, in der das Objekt angelegt wird.
     */
    @Override protected void addedToWorld​(World world)
    {
        super.addedToWorld​(world);
        symbol.SichtbarkeitSetzen(sichtbar);
    }
    
    /**
     * Setzt die Position (der linken oberen Ecke) des Rechtecks.
     * @param x x-Position der linken oberen Ecke
     * @param y y-Position der linken oberen Ecke
     */
    @Override public void setLocation(int x, int y)
    {
        double wb = winkel * Math.PI / 180.0;
        this.x = (int) (x - (50*Math.cos(wb) - 5*Math.sin (wb)));
        this.y = (int) (y + (50*Math.sin(wb) + 5*Math.cos (wb)));
        super.setLocation(x, y);
        
        if (symbol != null)
        {
            symbol.PositionSetzen(this.x, this.y);
        }
    }
        
    /**
     * Setzt den Drehwinkel des Textes.
     * Die Winkelangabe ist in Grad, positive Werte drehen im Uhrzeigersinn,
     * negative Werte drehen gegen den Uhrzeigersinn (mathematisch negativer Drehsinn).
     * @param winkel der (neue) Drehwinkel des Textes
     */
    @Override public void setRotation(int w)
    {
        winkel = -w;
        super.setRotation(w);
        if (symbol != null)
        {
            symbol.WinkelSetzen(winkel);
        }
    }
    
    /**
     * Setzt die Position (der Grundline) des Textes.
     * @param x x-Position der linken Seite der Grundlinie
     * @param y y-Position der Grundlinie
     */
    public void PositionSetzen(int x, int y)
    {
        this.x = x;
        this.y = y;
        symbol.PositionSetzen(x, y);
        double wb = winkel * Math.PI / 180.0;
        super.setLocation((int) (x + (50*Math.cos(wb) - 5*Math.sin (wb))), (int) (y - (50*Math.sin(wb) + 5*Math.cos (wb))));
    }
        
    /**
     * Setzt den aktuellen Text.
     * @param text der neue Text
     */
    public void TextSetzen (String text)
    {
        symbol.TextSetzen(text);
    }
    
    /**
     * Setzt die Größe des Textes.
     * @param größe die (neue) Textgröße
     */
    public void TextGrößeSetzen (int größe)
    {
        textgröße = größe;
        symbol.TextGrößeSetzen(größe);
    }
    
    /**
     * Vergrößert den Text.
     */
    public void TextVergrößern()
    {
        symbol.TextVergrößern();
        textgröße = (int) symbol.size;
    }
    
    /**
     * Verkleinert den Text.
     */
    public void TextVerkleinern()
    {
        symbol.TextVerkleinern();
        textgröße = (int) symbol.size;
    }
    
    /**
     * Verschiebt den Text um die angegebenen Werte.
     * @param deltaX Verschiebung in x-Richtung
     * @param deltaY Verschiebung in y-Richtung
     */
    public void Verschieben(int deltaX, int deltaY)
    {
        x += deltaX;
        y += deltaY;
        symbol.PositionSetzen(x, y);
        double wb = winkel * Math.PI / 180.0;
        super.setLocation((int) (x + (50*Math.cos(wb) - 5*Math.sin (wb))), (int) (y - (50*Math.sin(wb) + 5*Math.cos (wb))));
    }
    
    /**
     * Dreht den Text
     * @param grad Drehwinkel (mathematisch positiver Drehsinn) im Gradmass
     */
    public void Drehen(int grad)
    {
        winkel += grad;
        symbol.WinkelSetzen(winkel);
        super.setRotation(-winkel);
        double wb = winkel * Math.PI / 180.0;
        super.setLocation((int) (x + (50*Math.cos(wb) - 5*Math.sin (wb))), (int) (y - (50*Math.sin(wb) + 5*Math.cos (wb))));
    }
    
    /**
     * Setzt die Farbe des Textes.
     * Erlaubte Farben sind:
     * "weiß", "weiss", "rot", "grün", "gruen", "blau", "gelb",
     * "magenta", "cyan", "hellgelb", "hellgrün", "hellgruen",
     * "orange", "braun", "grau", "schwarz"
     * Alle anderen Eingaben werden auf die Farbe schwarz abgebildet.
     * @param farbe (neue) Farbe
     */
    public void FarbeSetzen (String farbe)
    {
        this.farbe = farbe;
        symbol.FarbeSetzen(farbe);
    }
        
    /**
     * Setzt den Drehwinkel des Textes.
     * Die Winkelangabe ist in Grad,positive Werte drehen gegen den Uhrzeigersinn,
     * negative Werte drehen im Uhrzeigersinn (mathematisch positiver Drehsinn).
     * @param winkel der (neue) Drehwinkel des Textes
     */
    public void WinkelSetzen (int winkel)
    {
        this.winkel = winkel;
        symbol.WinkelSetzen(winkel);
        super.setRotation(-winkel);
        double wb = winkel * Math.PI / 180.0;
        super.setLocation((int) (x + (50*Math.cos(wb) - 5*Math.sin (wb))), (int) (y - (50*Math.sin(wb) + 5*Math.cos (wb))));
    }
    
    /**
     * Schaltet die Sichtbarkeit des Textes ein oder aus.
     * Erlaubte Parameterwerte: true, false
     * @param sichtbar (neue) Sichtbarkeit des Textes
     */
    public void SichtbarkeitSetzen (boolean sichtbar)
    {
        this.sichtbar = sichtbar;
        symbol.SichtbarkeitSetzen(sichtbar);
    }
        
    /**
     * Entfernt den Text aus dem Zeichenfenster.
     */
    public void Entfernen ()
    {
        symbol.Entfernen();
        welt.removeObject(this);
    }
    
    /**
     * Bringt den Text eine Ebene nach vorn.
     */
    public void NachVornBringen ()
    {
        symbol.NachVornBringen();
    }
    
    /**
     * Bringt den Text in die vorderste Ebene.
     */
    public void GanzNachVornBringen ()
    {
        symbol.GanzNachVornBringen();
    }
    
    /**
     * Bringt den Text eine Ebene nach hinten.
     */
    public void NachHintenBringen ()
    {
        symbol.NachHintenBringen();
    }
    
    /**
     * Bringt den Text in die hinterste Ebene.
     */
    public void GanzNachHintenBringen ()
    {
        symbol.GanzNachHintenBringen();
    }
}
